# Generated by Django 4.2.21 on 2026-01-16 15:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_bb', '0111_alter_entityinfocache_as_of_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='complianceticket',
            name='details',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='entityinfocache',
            name='entity_id',
            field=models.BigIntegerField(db_index=True),
        ),
    ]
