# Generated by Django 4.2.21 on 2026-01-14 17:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_bb', '0110_bigbrotherconfig_hide_unaudited_users'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entityinfocache',
            name='as_of',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='entityinfocache',
            name='entity_id',
            field=models.IntegerField(db_index=True),
        ),
    ]
