# Generated by Django 4.2.21 on 2026-02-06 18:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aa_bb", "0113_bigbrotherconfig_discord_message_tracking"),
    ]

    operations = [
        migrations.AddField(
            model_name="bigbrotherconfig",
            name="webhook_app_name",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Name shown for Discord webhook messages sent by this app.",
                max_length=80,
                verbose_name="Webhook App Name",
            ),
        ),
    ]
