# Generated by Django 4.2.21 on 2026-01-16 17:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_bb', '0112_complianceticket_details_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='bigbrotherconfig',
            name='discord_message_tracking',
            field=models.BooleanField(default=True, help_text='Whether to track discord message activity (last_discord_message_at) for users', verbose_name='Discord Message Activity Tracking'),
        ),
    ]
